import tempfile
from pathlib import Path

import django
from django.test import TestCase
from moto import mock_s3

from caretaker.management.commands.create_backup import Command as CreateCommand
from caretaker.tests.utils import setup_bucket, upload_temporary_file


@mock_s3
class TestCreateBackup(TestCase):
    def setUp(self):
        setup_bucket(self)

        self.logger.info('Setup for create_backup')

        self.create_command = CreateCommand()

        django.setup()

    def tearDown(self):
        self.logger.info('Teardown for create_backup')
        pass

    def test_create(self):
        self.logger.info('Testing create_backup')

        with tempfile.TemporaryDirectory() as temporary_directory_name:
            temporary_directory_name = Path(
                temporary_directory_name).expanduser()

            # set up a temporary file
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            self.assertTrue(result == self.command.returns[1])

            # create a backup record including this directory
            json_file, data_file = self.create_command._create_backup(
                output_directory=temporary_directory_name,
                path_list=[temporary_directory_name]
            )

            self.assertTrue(json_file.exists())
            self.assertTrue(data_file.exists())



import tempfile

from django.test import TestCase
from moto import mock_s3

from caretaker.tests.utils import setup_bucket, upload_temporary_file


@mock_s3
class TestPushBackup(TestCase):
    def setUp(self):
        setup_bucket(self)
        self.logger.info('Setup for push_backup')

    def tearDown(self):
        self.logger.info('Teardown for push_backup')
        pass

    def test_push(self):
        self.logger.info('Testing push_backup')

        with tempfile.TemporaryDirectory() as temporary_directory_name:

            # set up a temporary file
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            self.assertTrue(result == self.command.returns[1])

            # run a second time and should not store the result
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            self.assertTrue(result == self.command.returns[2])

            # now test that when we add a new file it versions it
            with temporary_file.open('w') as out_file:
                out_file.write('test2')

            result = self.command._push_backup(
                backup_local_file=temporary_file, remote_key=self.json_key,
                s3_client=self.client, bucket_name=self.bucket_name)

            self.assertTrue(result == self.command.returns[1])



import tempfile

import boto3
from django.conf import settings
from django.core.management.base import BaseCommand
from caretaker.management.commands.create_backup import Command as CreateCommand
from caretaker.management.commands.push_backup import Command as PushCommand

from caretaker.main_utils import log


class Command(BaseCommand):
    """
    Installs cron tasks.
    """

    help = "Creates a backup set and pushes it to S3"

    def add_arguments(self, parser):
        parser.add_argument('--output-directory')
        parser.add_argument('-a', '--additional-files',
                            action='append', required=False)

    def handle(self, *args, **options):
        """
        Creates a backup set and pushes it to S3
        """
        s3 = boto3.client('s3',
                          aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                          aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

        self._run_backup(output_directory=options.get('output_directory'),
                         s3_client=s3,
                         bucket_name=settings.BACKUP_BUCKET,
                         path_list=options.get('additional_files'))

    @staticmethod
    def _run_backup(output_directory, data_file='data.json',
                    archive_file='media.zip', path_list=None,
                    s3_client=None, bucket_name=None):
        logger = log.get_logger('caretaker')

        s3 = s3_client

        if not path_list:
            path_list = []

        if not output_directory:
            logger.error('No output directory specified')
            return None, None

        # set up a temporary directory
        with tempfile.TemporaryDirectory() as temporary_directory_name:
            # create a local backup set in this temporary directory
            create_command = CreateCommand()

            json_file, zip_file = create_command._create_backup(
                output_directory=temporary_directory_name,
                path_list=path_list
            )

            # push the data
            push_command = PushCommand()
            push_command._push_backup(backup_local_file=json_file,
                                      remote_key=data_file,
                                      s3_client=s3, bucket_name=bucket_name)
            push_command._push_backup(backup_local_file=zip_file,
                                      remote_key=archive_file,
                                      s3_client=s3, bucket_name=bucket_name)

            logger.info('Pushed backups to remote store')
            return json_file, archive_file



import logging
from rich.logging import RichHandler


def get_logger(namespace='root', level=logging.INFO):
    format_string = "%(message)s"
    logging.basicConfig(
        level=level, format=format_string,
        datefmt="[%X]", handlers=[RichHandler()]
    )

    logger = logging.getLogger(namespace)
    logger.setLevel(level)

    return logger



import json
import tempfile
from pathlib import Path

import django
from django.test import TestCase
from moto import mock_s3

from caretaker.management.commands.list_backups import Command as ListCommand
from caretaker.management.commands.pull_backup import Command as PullCommand
from caretaker.management.commands.run_backup import Command as RunCommand
from caretaker.tests.utils import setup_bucket, upload_temporary_file, \
    file_in_zip


@mock_s3
class TestRunBackup(TestCase):
    def setUp(self):
        setup_bucket(self)

        self.logger.info('Setup for run_backup')

        self.run_command = RunCommand()
        self.pull_command = PullCommand()
        self.list_command = ListCommand()

        django.setup()

    def tearDown(self):
        self.logger.info('Teardown for run_backup')
        pass

    def test_run(self):
        self.logger.info('Testing run_backup')

        with tempfile.TemporaryDirectory() as temporary_directory_name:
            temporary_directory_name = Path(
                temporary_directory_name).expanduser()

            # set up a temporary file
            result, temporary_file = upload_temporary_file(
                test_class=self,
                temporary_directory_name=temporary_directory_name,
                contents=self.test_contents)

            self.assertTrue(result == self.command.returns[1])

            # create a backup record including this directory
            self.run_command._run_backup(
                output_directory=temporary_directory_name,
                path_list=[temporary_directory_name],
                bucket_name=self.bucket_name,
                s3_client=self.client
            )

            # now check that the files exist in S3
            # list the results to get a versionId of the SQL backup
            result = self.list_command._list_backups(
                remote_key=self.dump_key, bucket_name=self.bucket_name,
                s3_client=self.client
            )

            Path(temporary_directory_name / self.json_key).unlink(
                missing_ok=True)
            download_location = temporary_directory_name / self.json_key

            result = self.pull_command._pull_backup(
                backup_version=result[0]['VersionId'],
                remote_key=self.dump_key,
                bucket_name=self.bucket_name,
                s3_client=self.client,
                out_file=download_location
            )

            with result.open('r') as in_file:
                response = in_file.read()

                try:
                    json.loads(response)
                except json.JSONDecodeError:
                    self.fail('The stored object could not be JSON decoded')

            # now check the archive zip
            result = self.list_command._list_backups(
                remote_key=self.data_key, bucket_name=self.bucket_name,
                s3_client=self.client
            )

            Path(temporary_directory_name / self.data_key).unlink(
                missing_ok=True)
            download_location = temporary_directory_name / self.data_key

            result = self.pull_command._pull_backup(
                backup_version=result[0]['VersionId'],
                remote_key=self.data_key,
                bucket_name=self.bucket_name,
                s3_client=self.client,
                out_file=download_location
            )

            self.assertTrue(file_in_zip(zip_file=result,
                                        filename=self.json_key))



import tempfile
from pathlib import Path

from django.test import TestCase
from moto import mock_s3

from caretaker.management.commands.get_terraform import Command as \
    TerraformCommand
from caretaker.tests.utils import setup_bucket


@mock_s3
class TestTerraformOutput(TestCase):
    def setUp(self):
        setup_bucket(self)

        self.logger.info('Setup for Terraform test')

        self.command = TerraformCommand()

    def tearDown(self):
        self.logger.info('Teardown Terraform test')
        pass

    def test_terraform(self):
        self.logger.info('Running Terraform test')
        with tempfile.TemporaryDirectory() as temporary_directory_name:
            self.command._generate_terraform(
                output_directory=temporary_directory_name)

            self.assertTrue(
                (Path(temporary_directory_name) / 'main.tf').exists())
            self.assertTrue(
                (Path(temporary_directory_name) / 'output.tf').exists())



from pathlib import Path

import boto3
from botocore.exceptions import ClientError
from django.conf import settings
from django.core.management.base import BaseCommand

from caretaker.main_utils import log


class Command(BaseCommand):
    """
    Installs cron tasks.
    """

    help = "Pulls a specific backup from the S3 store"

    def add_arguments(self, parser):
        parser.add_argument('--backup-version')
        parser.add_argument('--out-file')
        parser.add_argument('--remote-key',
                            default='backup.sql')

    def handle(self, *args, **options):
        """
        Pulls a backup from S3
        """
        s3 = boto3.client('s3',
                          aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                          aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

        self._pull_backup(out_file=options.get('backup_local_file'),
                          remote_key=options.get('remote_key'),
                          s3_client=s3, bucket_name=settings.BACKUP_BUCKET,
                          backup_version=options.get('backup_version'))

    @staticmethod
    def _pull_backup(backup_version, out_file, remote_key, s3_client,
                     bucket_name):
        logger = log.get_logger('caretaker')

        s3 = s3_client

        out_file = Path(out_file).expanduser()

        try:
            s3.download_file(
                Filename=str(out_file),
                Bucket=bucket_name,
                Key=remote_key,
                ExtraArgs={'VersionId': backup_version}
            )

            logger.info('Saved version {} of {} to {}'.format(
                backup_version,
                remote_key,
                out_file
            ))

            return out_file
        except ClientError:
            logger.error('Unable to download version {} of {} to {}'.format(
                backup_version,
                remote_key,
                out_file
            ))
            return None



import filecmp
import tempfile
from pathlib import Path

import boto3
from botocore.exceptions import ClientError
from django.conf import settings
from django.core.management.base import BaseCommand

from caretaker.main_utils import log


class Command(BaseCommand):
    """
    Installs cron tasks.
    """

    help = "Pushes the backup to the S3 store"

    returns = {0: 'FAILED',
               1: 'STORED',
               2: 'IDENTICAL'}

    def add_arguments(self, parser):
        parser.add_argument('--backup-local-file',
                            default='~/backup.sql')
        parser.add_argument('--remote-key',
                            default='backup.sql')

    def handle(self, *args, **options):
        """
        Pushes the backup to S3
        """
        s3 = boto3.client('s3',
                          aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                          aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

        self._push_backup(backup_local_file=options.get('backup_local_file'),
                          remote_key=options.get('remote_key'),
                          s3_client=s3, bucket_name=settings.BACKUP_BUCKET)

    def _push_backup(self, backup_local_file, remote_key, s3_client,
                     bucket_name):
        logger = log.get_logger('caretaker')

        backup_local_file = Path(backup_local_file).expanduser()

        s3 = s3_client

        # download the latest version of the backup to see if it's the same
        # as the local file
        try:
            with tempfile.TemporaryDirectory() as tmp:
                path = Path(tmp) / 'latest'

                s3.download_file(Filename=str(path), Bucket=bucket_name,
                                 Key=remote_key)

                if filecmp.cmp(path, backup_local_file):
                    logger.info('Latest backup is equal to remote S3 version')
                    return self.returns[2]

        except ClientError:
            logger.error('There was a problem comparing the previous version '
                         'of this log with the stored version. This is not '
                         'a fatal error.')
            pass

        try:
            # upload the latest version to S3
            s3.upload_file(Filename=str(backup_local_file),
                           Bucket=bucket_name, Key=remote_key)

            logger.info('Backup {} stored as {}'.format(
                backup_local_file, remote_key))
        except ClientError:
            logger.error('There was a problem storing the backup.')
            return self.returns[0]

        return self.returns[1]



from io import StringIO
from pathlib import Path

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand

from caretaker.main_utils import log
from caretaker.main_utils.zip import create_zip_file


class Command(BaseCommand):
    """
    Installs cron tasks.
    """

    help = "Creates a local backup set"

    def add_arguments(self, parser):
        parser.add_argument('--output-directory')
        parser.add_argument('-a', '--additional-files',
                            action='append', required=False)

    def handle(self, *args, **options):
        """
        Creates a local backup set
        """

        self._create_backup(output_directory=options.get('output_directory'),
                            path_list=options.get('additional_files'))

    @staticmethod
    def _create_backup(output_directory, data_file='data.json',
                       archive_file='media.zip', path_list=None):
        logger = log.get_logger('caretaker')

        if not output_directory:
            logger.error('No output directory specified')
            return None, None

        output_directory = Path(output_directory).expanduser()

        # create the directory if needed
        output_directory.mkdir(parents=True, exist_ok=True)

        # setup redirect so that we can pipe the output of dump data to
        # our output file
        buffer = StringIO()
        call_command('dumpdata', stdout=buffer)
        buffer.seek(0)

        with (output_directory / data_file).open('w') as out_file:
            out_file.write(buffer.read())
            logger.info('Wrote {}'.format(data_file))

        # now create a tarball of the media directory and any others specified
        path_list = [] if not path_list else path_list
        path_list = list(set(path_list))

        if hasattr(settings, 'MEDIA_ROOT') and \
                settings.MEDIA_ROOT and settings.MEDIA_ROOT not in path_list:
            path_list.append(settings.MEDIA_ROOT)

        if hasattr(settings, 'ADDITIONAL_BACKUP_PATHS') \
                and settings.ADDITIONAL_BACKUP_PATHS \
                and settings.ADDITIONAL_BACKUP_PATHS not in path_list:

            path_list.extend(settings.ADDITIONAL_BACKUP_PATHS)

        path_list_final = [Path(path).expanduser().resolve(strict=True)
                           for path in path_list]

        zip_file = create_zip_file(
            input_paths=path_list_final,
            output_file=Path(output_directory / archive_file)
        )

        logger.info('Wrote {} ({})'.format(archive_file, zip_file))

        return output_directory / data_file, zip_file



import os
from pathlib import Path

from django.conf import settings
from django.core.management.base import BaseCommand
from django.template import Template, Context

from caretaker.main_utils import log


class Command(BaseCommand):
    """
    Installs cron tasks.
    """

    help = "Gets a terraform setup configuration"

    @staticmethod
    def _output_terraform_file(output_directory, terraform_file, file_name):
        with terraform_file.open('r') as in_file:
            # render the terraform file into a template
            t = Template(in_file.read())
            c = Context({"bucket_name": settings.BACKUP_BUCKET})
            rendered = t.render(c)

            # create the file structure
            output_directory.mkdir(parents=True, exist_ok=True)

            # write the terraform file to this directory
            with (output_directory / file_name).open('w') as out_file:
                out_file.write(rendered)

    @staticmethod
    def _generate_terraform(output_directory):
        logger = log.get_logger('caretaker')
        output_directory = Path(output_directory).expanduser()

        # configure file paths
        terraform_dir = Path(
            os.path.realpath(__file__)).parent.parent.parent / 'terraform'

        terraform_file = Path(terraform_dir / 'main.tf')
        terraform_output_file = Path(terraform_dir / 'output.tf')

        Command._output_terraform_file(
            output_directory, terraform_file, 'main.tf')
        Command._output_terraform_file(
            output_directory, terraform_output_file, 'output.tf')

        logger.info('Terraform files were written to {}'.format(
            output_directory))

    def add_arguments(self, parser):
        parser.add_argument('--output-directory',
                            default='~/terraform_configuration')

    def handle(self, *args, **options):
        """
        Produces a Terraform setup configuration
        """
        self._generate_terraform(options.get('output_directory'))



import boto3
import humanize
from django.conf import settings
from django.core.management.base import BaseCommand
from caretaker.main_utils import log


class Command(BaseCommand):
    """
    Installs cron tasks.
    """

    help = "Lists available backups"

    def add_arguments(self, parser):
        parser.add_argument('--remote-key',
                            default='backup.sql')

    def handle(self, *args, **options):
        """Lists available backups"""

        s3 = boto3.client('s3',
                          aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                          aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

        self._list_backups(s3_client=s3, remote_key=options.get('remote_key'),
                           bucket_name=settings.BACKUP_BUCKET)

    @staticmethod
    def _list_backups(remote_key, s3_client, bucket_name):
        logger = log.get_logger('caretaker')

        s3 = s3_client

        versions = s3.list_object_versions(Bucket=bucket_name,
                                           Prefix=remote_key)

        if versions and 'Versions' in versions:
            for item in versions['Versions']:
                logger.info('Backup from {}: {} [{}]'.format(
                    item['LastModified'],
                    item['VersionId'],
                    humanize.naturalsize(item['Size'])
                ))

            return versions['Versions']

        return None



from pathlib import Path
from zipfile import ZipFile, ZIP_DEFLATED


def create_zip_file(input_paths, output_file):
    with ZipFile(output_file.expanduser(), 'w', ZIP_DEFLATED) as zf:
        for directory in input_paths:
            for file in directory.rglob('*'):
                zf.write(file, file.relative_to(directory.parent))

    return Path(output_file)



import os
import zipfile
from pathlib import Path

import boto3

from caretaker.main_utils import log
from caretaker.management.commands.push_backup import Command as PushCommand

DEV_NULL = open(os.devnull, "w")


def setup_bucket(test_class):
    """
    Generic class to set up tests that require push functionality
    :param test_class: the class to mutate
    :return: the test class
    """
    test_class.command = PushCommand()
    test_class.json_key = 'test.json'
    test_class.dump_key = 'data.json'
    test_class.data_key = 'media.zip'
    test_class.test_contents = 'test'

    test_class.logger = log.get_logger('caretaker')

    test_class.client = boto3.client(
        's3',
        region_name='us-east-1',
        aws_access_key_id='fake_access_key',
        aws_secret_access_key='fake_secret_key',
    )

    test_class.bucket_name = 'a_test_bucket'
    test_class.client.create_bucket(Bucket=test_class.bucket_name)
    test_class.client.put_bucket_versioning(
        Bucket=test_class.bucket_name,
        ChecksumAlgorithm='CRC32',
        VersioningConfiguration={
            'Status': 'Enabled'
        }
    )

    return test_class


def upload_temporary_file(test_class, temporary_directory_name, contents):
    temporary_file = Path(temporary_directory_name) / test_class.json_key

    with temporary_file.open('w') as out_file:
        out_file.write(contents)

    # run the first time to store the result
    result = test_class.command._push_backup(
        backup_local_file=temporary_file, remote_key=test_class.json_key,
        s3_client=test_class.client, bucket_name=test_class.bucket_name)

    return result, temporary_file


def file_in_zip(zip_file, filename):
    with zipfile.ZipFile(zip_file, 'r') as zf:
        name_list = [filename.split('/')[-1]
                     for filename in zf.namelist()]
        return filename in name_list



import os

from crontab import CronTab
from django.conf import settings
from django.core.management.base import BaseCommand

from caretaker.main_utils import log


def find_job(tab, comment):
    for job in tab:
        if job.comment == comment:
            return job
    return None


class Command(BaseCommand):
    """
    Installs cron tasks.
    """

    help = "Installs cron tasks."

    def add_arguments(self, parser):
        parser.add_argument('--action', default="")

    def handle(self, *args, **options):
        """Installs Cron jobs
        """
        self._install_cron(job_name=settings.BACKUP_BUCKET,
                           action=options.get('action'),
                           base_dir=settings.BASE_DIR)

    @staticmethod
    def _install_cron(job_name, action, base_dir):
        logger = log.get_logger('caretaker')
        tab = CronTab(user=True)
        virtualenv = os.environ.get('VIRTUAL_ENV', None)

        jobs = [
            {
                'name': 'caretaker_sync_{}_job'.format(job_name),
                'task': 'run_backup',
            },
        ]

        for job in jobs:
            current_job = find_job(tab, job['name'])

            if not current_job:
                django_command = "{0}/manage.py {1}".format(base_dir,
                                                            job['task'])
                command = '%s/bin/python3 %s' % (virtualenv, django_command)

                cron_job = tab.new(command, comment=job['name'])
                cron_job.setall("15 0 * * *")

            else:
                logger.info("{name} cron job already exists.".format(
                    name=job['name']))

        if action == 'test':
            logger.info(tab.render())
        elif action == 'quiet':
            pass
        else:
            tab.write()



import io

import boto3
import humanize
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.http import StreamingHttpResponse
from django.shortcuts import render


@staff_member_required
def list_backups(request):
    s3 = boto3.client('s3',
                      aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                      aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

    sql_versions = fetch_versions(s3, 'data.json')
    data_versions = fetch_versions(s3, 'media.zip')

    template = 'backup_list.html'

    context = {'sql_versions': sql_versions,
               'data_versions': data_versions,
               }

    return render(
        request,
        template,
        context,
    )


@staff_member_required
def download_backup(request, backup_type, version_id):
    s3 = boto3.client('s3',
                      aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
                      aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)

    if backup_type == 'sql':
        key = 'data.json'
    else:
        key = 'media.zip'

    response_object = io.BytesIO()
    s3.download_fileobj(Bucket=settings.BACKUP_BUCKET, Key=key,
                        Fileobj=response_object,
                        ExtraArgs={'VersionId': version_id})
    response_object.seek(0)

    resp = StreamingHttpResponse(streaming_content=response_object)
    resp['Content-Disposition'] = 'attachment; ' \
                                  'filename="{}-{}"'.format(version_id, key)

    return resp


def fetch_versions(s3, key):
    versions = s3.list_object_versions(
        Bucket=settings.BACKUP_BUCKET, Prefix=key)

    sql_versions = []

    for item in versions['Versions']:
        sql_versions.append({'lastmodified': item['LastModified'],
                             'versionid': item['VersionId'],
                             'size': humanize.naturalsize(item['Size'])})

    return sql_versions
